<?php

namespace Drupal\services_token\Authentication\Provider;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Authentication\AuthenticationProviderInterface;
use Drupal\Core\Authentication\AuthenticationProviderChallengeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\services_token\RealmResolverInterface;
use Drupal\services_token\SecurityKeyInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

/**
 * Services token authentication provider.
 */
class TokenAuth implements AuthenticationProviderInterface, AuthenticationProviderChallengeInterface {

  /**
   * Security key.
   *
   * @var \Drupal\services_token\SecurityKeyInterface
   */
  protected $securityKey;

  /**
   * Realm resolver.
   *
   * @var \Drupal\services_token\RealmResolverInterface
   */
  protected $realmResolver;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a HTTP basic authentication provider object.
   *
   * @param \Drupal\services_token\SecurityKeyInterface $security_key
   *   The security key
   * @param \Drupal\services_token\RealmResolverInterface $realm_resolver
   *   The realm resolver.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(
    SecurityKeyInterface $security_key,
    RealmResolverInterface $realm_resolver,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->securityKey = $security_key;
    $this->realmResolver = $realm_resolver;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(Request $request) {
    $token = $request->headers->get('PHP_AUTH_USER');
    $password = $request->headers->get('PHP_AUTH_PW');
    return isset($token) && empty($password);
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate(Request $request) {
    $token = $request->headers->get('PHP_AUTH_USER');
    $realm = $this->realmResolver->resolveRealm();

    if ($this->securityKey->verify($token, $realm)) {
      [ $huid ] = explode('.', $token, 2);
      $uid = hexdec($huid);
      return $this->entityTypeManager->getStorage('user')->load($uid);
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function challengeException(Request $request, \Exception $previous) {
    $realm = $this->realmResolver->resolveRealm();
    $challenge = new FormattableMarkup('Basic realm="@realm"', [
      '@realm' => $realm,
    ]);

    return new UnauthorizedHttpException(
      (string) $challenge,
      'No authentication credentials provided.',
      $previous
    );
  }

}
