<?php

namespace Drupal\services_token;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Site\Settings;

/**
 * Provides methods to generate and check the HMAC for protected downloads
 */
class SecurityKey implements SecurityKeyInterface {

  /**
   * HMAC key.
   *
   * @var string
   */
  protected $hmacKey;

  /**
   * The request time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a services token security key service.
   *
   * @param \Drupal\Component\Datetime\TimeInterface
   *   The request time service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface
   *   The module handler.
   */
  function __construct(
    TimeInterface $time,
    ModuleHandlerInterface $module_handler,
    $hmac_key = NULL
  ) {
    $this->hmacKey = $hmac_key ?? Settings::get('services_token_private_key', \Drupal::service('private_key')->get() . Settings::getHashSalt());
    $this->time = $time;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  function verify(string $supplied_token, string $realm): bool {
    $result = FALSE;

    if (substr_count($supplied_token, '.') === 2) {
      [ $huid, $hexp ] = explode('.', $supplied_token);
      $uid = hexdec($huid);
      $expire = hexdec($hexp);

      $computed_token = $this->generate($uid, $expire, $realm);

      $result = hash_equals($computed_token, $supplied_token) &&
        $this->time->getRequestTime() < $expire;
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  function generate(int $uid, int $expire, string $realm): string {
    $properties = (array) $this->moduleHandler->invokeAll(
      'services_token_properties',
      [ $uid, $realm ]
    );

    $this->moduleHandler->alter(
      'services_token_properties',
      $properties,
      $uid,
      $realm
    );

    ksort($properties);
    $data = serialize($properties);

    $hmac = Crypt::hmacBase64($uid . $data . $expire, $this->hmacKey);

    return implode(".", [dechex($uid), dechex($expire), $hmac]);
  }

}
