<?php

namespace Drupal\services_token;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Site\Settings;

class TokenGenerator implements TokenGeneratorInterface {

  /**
   * Default services token TTL.
   */
  const TOKEN_TTL_DEFAULT = 2592000;

  /**
   * Security key.
   *
   * @var \Drupal\services_token\SecurityKeyInterface
   */
  protected $securityKey;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Realm resolver.
   *
   * @var \Drupal\services_token\RealmResolverInterface
   */
  protected $realmResolver;

  /**
   * Request time.
   *
   * @var \Drupal\Component\Datetime\Time
   */
  protected $time;

  /**
   * Settings.
   *
   * @var \Drupal\Core\Site\Settings
   */
  protected $settings;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Creates a new services token generator.
   *
   * @param \Drupal\services_token\SecurityKeyInterface $security_key
   *   The security key
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\services_token\RealmResolverInterface $realm_resolver
   *   The realm resolver.
   * @param \Drupal\Component\Datetime\Time $time
   *   The current request time.
   * @param \Drupal\Core\Site\Settings $settings
   *   The settings object.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface
   *   The module handler.
   */
  public function __construct(
    SecurityKeyInterface $security_key,
    AccountInterface $current_user,
    RealmResolverInterface $realm_resolver,
    TimeInterface $time,
    Settings $settings,
    ModuleHandlerInterface $module_handler
  )
  {
    $this->securityKey = $security_key;
    $this->currentUser = $current_user;
    $this->realmResolver = $realm_resolver;
    $this->time = $time;
    $this->settings = $settings;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function generate(int $uid = NULL, int $expire = NULL, string $realm = NULL): array {
    if (!isset($uid)) {
      $uid = $this->currentUser->id();
    }

    if (!isset($expire)) {
      $ttl = $this->settings->get('services_token_ttl', self::TOKEN_TTL_DEFAULT);
      $expire = $this->time->getRequestTime() + $ttl;
    }

    if (!isset($realm)) {
      $realm = $this->realmResolver->resolveRealm();
    }

    $this->moduleHandler->alter('services_token_expires', $expire, $uid, $realm);

    $resource = [
      'expires' => date(DATE_ISO8601, $expire),
      'token' => $this->securityKey->generate($uid, $expire, $realm),
    ];

    $this->moduleHandler->alter('services_token_create', $resource, $uid, $realm);

    return $resource;
  }

}
